/*Program provided by Tabitha Garwe, PhD and David M. Thompson, PhD */
/*Adapted from BSE 6193 Methods in Clinical Epidemiology*/;

/*Read into SAS the provided dataset (injsev1.sas7bdat), which is similar to the one used by Smith et al.'*/;

*libname clinepi '';
Data one;
set clinepi.injsev1;
if dstatus='7' then mortality=1; 
else mortality=0;
if age le 45 then agegrp=1; else agegrp=0;
run;

proc format;
  value mortality 1="Died" 0="Survived";
  value agegrp 1="45 or younger" 0="Older than 45";
run;


/*Determine total sample size, number who died, number who survived, and the prevalence of mortality in this sample.*/;
proc freq data=one;
tables mortality/nocum ;
format mortality mortality.;
run;

/*Part I a. The idea here is to show how the distribution of ISS scores overlap for two groups of subjects 
with penetrating injuries: a group who died and a group who survived.
The overlap shows that the ISS' ability to predict death is not perfect;
some misclassification is inevitable.*/

data histogram;
  set one;
  if mortality=1 then issd=iss;
  else if mortality=0 then issn=iss;
run;
proc sgplot data=histogram;
       density issd / type=kernel (c=2) lineattrs=(color=red) ;
       density issn / type=kernel (c=2) lineattrs=(color=green) ;
       histogram issd / transparency=0.75 fillattrs=(color=red)
	                    binstart = 0 binwidth = 10 scale=percent showbins
                        legendlabel='ISS for those who died' name='one';
       histogram issn / transparency=0.75 fillattrs=(color=green)
                        binstart = 0 binwidth = 10 scale =percent showbins
                        legendlabel='ISS for survivors' name='two';
       keylegend 'two' 'one'  / down=2 location=outside position=bottom;
       xaxis label="Injury Severity Score (ISS)";
run;

/*Part I  b: The ISS (and the NISS) are both multi-level or continuous measures.  Use SAS to create a new variable 
that dichotomizes the ISS at different cutpoints and explore how the ISS sensitivity and specificity differs at these cutpoints.*/;  

data isscut;
set one;
if ISS >=9 then ISS9=1; else ISS9=0;
if ISS >=16 then ISS16=1; else ISS16=0;
if ISS >=25 then ISS25=1; else ISS25=0;
if ISS >=35 then ISS35=1; else ISS35=0;
proc sort data=isscut; by descending iss9 descending  mortality; run;

ods rtf file='diffcut.rtf';
PROC FREQ order=data data=isscut;
TABLES (ISS9 ISS16 ISS25 ISS35) *Mortality/nopercent norow;
run;
ods rtf close;

/* Part I c: ISS Test Performance in predicting mortality*/;


proc logistic descending data=one;
   model mortality = iss  / lackfit risklimits;
  run;

ods graphics on;
proc logistic descending data=one plots=roc(id= cutpoint);;
   model mortality = iss  /  outroc=t1;
   roc; roccontrast;
    output out=t2 p=_prob_;
run;
ods graphics off;

/* Part I d: Examine the ROC curve */

/*PROC LOGISTIC's default commands cannot label the ROC curve
  with the cutpoints, so the next part of the program
  does that (FOR THE VARIABLE 'ISS').

Run the next segment of the program all the way through
the SGPLOT step, and examine the ROC curve that is produced
with labeled cutpoints.
*/

proc sort data=t1 (where=(_source_="Model"));;
  by _prob_;
proc sort data=t2;
  by _prob_;
run;

/*The next data step assembles information from two 
other SAS datasets, T1 and T2, for production of the
ROC curve that is labeled with cutpoins. */;


data graph;
  merge t1 (keep= _source_ _prob_ _sensit_ _1mspec_)
        t2 (keep=_prob_ iss);
  by _prob_;
  if first._prob_ and _prob_ ne .;
/*Create new variable M for odd numbered cutpoints*/
  *if mod(iss,2)=1 or iss=16 then m=iss;
  IF ISS IN (9, 16, 25, 35) THEN M=ISS;
run;

proc sgplot data=graph;
  xaxis label="1-Specificity" values=(0 to 1 by 0.1)
        labelattrs=(family="Albany AMT" size=9 weight=bold) ;
  yaxis label="Sensitivity" values=(0 to 1 by 0.1)
        labelattrs=(family="Albany AMT" size=9 weight=bold);
/*Mark and connect points on ROC curve, and label them
  using variable M.*/
  series y=_sensit_ x=_1mspec_ 
      /datalabel=m markers
       markerattrs=(color=green symbol=circle size=12)
       lineattrs = (color=green pattern=solid thickness=2)
       datalabelattrs=(family="Albany AMT" size=12 weight=bold);
run;

/*Comparison of ISS and NISS in Predicting Mortality*/;


/*Part II a:  DEPENDENT ROC CURVES*/;

ods graphics on;
proc logistic descending data=one plots=roc(id= cutpoint);
   model mortality = iss niss / nofit outroc=t1;
   roc 'ISS' iss;
   roc 'NISS' niss;
   roccontrast reference('ISS') / estimate e;
run;
ods graphics off;

/*Part II b: Added Prognostic Value*/;
ods graphics on;
proc logistic descending data=one plots=roc(id= cutpoint);
   model mortality = iss age / nofit outroc=t1;
   roc 'ISS' iss;
   roc 'ISS and Age' iss age;
   roccontrast reference('ISS') / estimate e;
run;
ods graphics off;

/*Part II C: INDEPENDENT ROC CURVES*/;

/*ROC curves from models fit to two or more independent groups of observations 
are not dependent and therefore cannot be compared using the ROC and ROCCONTRAST statements in PROC LOGISTIC. 

Those statements compare dependent curves, 
such as when comparing competing models fit to the same set of observations*/;

proc freq data=one;
tables agegrp*mortality;
run;

/*Use logistic regression to fit ROC curves to either of
  the two independent groups defined by age*/
 ods graphics on;
 proc logistic descending data=one;
    where agegrp=1;
    model mortality = niss;
    roc;
    ods output ROCassociation=AUCy ROCCurve=ROCy;
 run;

 proc logistic descending data=one;
    where agegrp=0;
    model mortality = niss;
    roc;
    ods output ROCassociation=AUCg ROCCurve=ROCg;
run;

/*combine ROC data from the two models*/
data twoplots;
   set ROCy ROCg;
run;
/*overlaid ROC plots*/
proc sgplot data=twoplots;
  series y=_sensit_ x=_1mspec_ / group=_roc_;
run;

/*Test of difference in AUC for two independent groups*/
data roctest2;
  if _n_=1 then set aucy (where=(rocmodel="Model")
                          rename=(area=AUC1 stderr=s1));
  set aucg (where=(rocmodel="Model")
            rename=(area=AUC2 stderr=s2));
/*chi-square test for independent proportions*/
  Chisq=(auc1 - auc2)**2/(s1**2 + s2**2);
  Prob=1-probchi(Chisq,1); 
  format Prob pvalue6.; 
  Test="AUC1 - AUC2 = 0";
run;
proc print noobs data=roctest2;
   var AUC1 AUC2 Test Chisq Prob;
run;


/*THRESHOLD SELECTION*/;

/* Part III: Threshold Selection Using ISS*/

data maxsnsp;
set graph;
spec=1 - (_1mspec_);
maxsum=_sensit_ + 1 - (_1mspec_);
Youden= _sensit_ + spec - 1;

/* Cost-Minimization Criterion - maximizes function Sn - m(1-Sp)*/;
prior=0.92/0.08;
costratio = 0.14;
m=costratio*prior;
optim=_sensit_ -m*(_1mspec_);
run;

/* Part III A: Cutpoint that maximizes the sum of sensitivity and specificity for predicting mortality*/;
proc sort data= maxsnsp; by descending maxsum;run; 
proc print data=maxsnsp;
var ISS _sensit_ spec maxsum;
run; 
proc sort data= maxsnsp; by descending youden;run; 
proc print data=maxsnsp;
var ISS _sensit_ spec youden;
run; 



/* Part III B: Cost-Minimization Criterion*/;
proc sort data= maxsnsp; by descending optim;run; 
proc print data=maxsnsp;
var ISS _sensit_ spec costratio optim;
run; 



/* TIME - PERMITTING*/;

/* Likelihood ratios for intervals*/
/*The slope of the ROC over an interval reflects the LR
for the response defined by the interval.

By creating leading and lagging variables for 
Sn and (1-Sp), we can create a kind of "moving average"
and calculate the curve's slope and the LR(r) 
over overlapping intervals for the ISS.  

This data step uses the previously merged dataset "graph."

The program uses the global or macro variable HW 
to specify the "halfwidth" of the interval
that centers on a value for the ISS.  We can choose the
width of the interval by experimenting with different values.
The interval should be large enough to 
provide "smooth" rather than discontinuous estimates of
the ROC curve's slope. However, the intervals should not be so small [or narrow] 
that the resultant smoothing produces distorted estimates */

/*Specify here the "halfwidth of the interval.
The program calculates the local slope for the interval
that contains the specified number of observations
above and below a particular ISS score.  */
  %let hw=4; 
proc expand /*calculates leading and lagging variables*/
  data=graph out=lr method=none;
  id iss;
  convert _sensit_= snlead / transform=( lead &hw );
  convert _sensit_= snlag / transform=( lag &hw );
  convert _1mspec_ = onemsplead  / transform=( lead &hw );
  convert _1mspec_ = onemsplag / transform=( lag &hw );
  convert iss= top / transform=(lead &hw);
  convert iss = bottom / transform=(lag &hw);
run;
data lr;
  set lr;
  interval= bottom||"<= ISS <"||left(top);
  LR=(snlag-snlead) / (onemsplag-onemsplead);
run;
/*The LR is equal to the slope of the ROC curve 
  in various regions*/
proc print noobs data=lr;
  where lr ne .;
  var iss interval lr;
run;
